==> planner-4.0.5-sources.jar: com/xebialabs/deployit/deployment/orchestrator/CompositePackageOrchestrators.scala <==
package com.xebialabs.deployit.deployment.orchestrator

import java.util

import com.xebialabs.deployit.engine.spi.orchestration.Orchestrator
import com.xebialabs.deployit.plugin.api.deployment.specification.{Delta, DeltaSpecification}
import com.xebialabs.deployit.plugin.api.udm.{DeploymentPackage, Deployable, CompositePackage, Version}

import com.xebialabs.deployit.engine.spi.orchestration.Orchestrations._
import com.xebialabs.deployit.deployment.orchestrator.DescriptionHelper._

import collection.convert.wrapAll._
import scala.collection.mutable

trait PackageOrchestratorBase extends OrchestratorUtil {
  
  def matchDeployableIdByPackageId(delta:Delta, version:Version) =
    getDeployed(delta).getDeployable.asInstanceOf[Deployable].getId.startsWith(version.getId)

  def getOrchestrations(spec: DeltaSpecification) = spec.getDeployedApplication.getVersion match {
    case comp: CompositePackage =>
      val orderForOperation = getOrderForInt(spec.getOperation)
      val previousVersions: util.List[Version] = Option(spec.getPreviousDeployedApplication).map(_.getVersion.asInstanceOf[CompositePackage].getPackages).getOrElse(new util.ArrayList[Version]())
      val indexedPackages: List[(Version, Int)] = (getAllDeploymentPackages(comp) ++ previousVersions).toList.zipWithIndex
      val versionToDeltas: Map[(Version, Int), mutable.Buffer[Delta]] = spec.getDeltas.groupBy(delta => indexedPackages.find(version => matchDeployableIdByPackageId(delta, version._1)).get)
      versionToDeltas.toList.sortBy(_._1._2)(orderForOperation)
        .map { case (v, ds) => interleaved(getDescriptionForPackage(spec.getOperation, v._1), ds)}
    case _ => List(new DefaultOrchestrator().orchestrate(spec))
  }

  private def getAllDeploymentPackages(comp: CompositePackage): Seq[Version] = {
    comp.getPackages.flatMap {
      case depl: DeploymentPackage => List(depl)
      case comp: CompositePackage => getAllDeploymentPackages(comp)
    }
  }
}

@Orchestrator.Metadata (name = "sequential-by-composite-package", description = "The sequential by composite package orchestrator")
class SequentialByPackageOrchestrator extends Orchestrator with PackageOrchestratorBase {
  def orchestrate(spec: DeltaSpecification) = serial(getDescriptionForSpec(spec), getOrchestrations(spec))
}

@Orchestrator.Metadata (name = "parallel-by-composite-package", description = "The parallel by composite package orchestrator")
class ParallelByPackageOrchestrator extends Orchestrator with PackageOrchestratorBase {
  def orchestrate(spec: DeltaSpecification) = parallel(getDescriptionForSpec(spec), getOrchestrations(spec))
}

/***
  *  Deprecated
  */
@Orchestrator.Metadata (name = "composite-package", description = "The sequential by package orchestrator (Deprecated)")
@Deprecated
class CompositePackageOrchestrator extends SequentialByPackageOrchestrator

==> planner-4.5.0-sources.jar: com/xebialabs/deployit/deployment/orchestrator/CompositePackageOrchestrators.scala <==
package com.xebialabs.deployit.deployment
package orchestrator

import com.xebialabs.deployit.engine.spi.orchestration.Orchestrator
import com.xebialabs.deployit.plugin.api.deployment.specification.{Delta, DeltaSpecification}
import com.xebialabs.deployit.plugin.api.udm.{Deployable, CompositePackage, Version}

import com.xebialabs.deployit.engine.spi.orchestration.Orchestrations._
import com.xebialabs.deployit.deployment.orchestrator.DescriptionHelper._

import com.xebialabs.platform._
import collection.convert.wrapAll._

trait PackageOrchestratorBase extends OrchestratorUtil {
  
  def matchDeployableIdByPackageId(delta:Delta, version:Version) =
    delta.correctDeployed.getDeployable.asInstanceOf[Deployable].getId.startsWith(version.getId)
  
  def getOrchestrations(spec: DeltaSpecification) = spec.getDeployedApplication.getVersion match {
    case comp: CompositePackage => {
      val orderForOperation = getOrderForInt(spec.getOperation)
      val indexedPackages = comp.getPackages.toList.zipWithIndex
      val versionToDeltas = spec.getDeltas
        .groupBy(delta => indexedPackages.find(version => matchDeployableIdByPackageId(delta, version._1)).get)
      versionToDeltas.toList.sortBy(_._1._2)(orderForOperation)
        .map { case (v, ds) => interleaved(getDescriptionForPackage(spec.getOperation, v._1), ds)}
    }
    case _ => List(new DefaultOrchestrator().orchestrate(spec))
  }
}

@Orchestrator.Metadata (name = "sequential-by-composite-package", description = "The sequential by composite package orchestrator")
class SequentialByPackageOrchestrator extends Orchestrator with PackageOrchestratorBase {
  def orchestrate(spec: DeltaSpecification) = serial(getDescriptionForSpec(spec), getOrchestrations(spec))
}

@Orchestrator.Metadata (name = "parallel-by-composite-package", description = "The parallel by composite package orchestrator")
class ParallelByPackageOrchestrator extends Orchestrator with PackageOrchestratorBase {
  def orchestrate(spec: DeltaSpecification) = parallel(getDescriptionForSpec(spec), getOrchestrations(spec))
}

/***
  *  Deprecated
  */
@Orchestrator.Metadata (name = "composite-package", description = "The sequential by package orchestrator (Deprecated)")
@Deprecated
class CompositePackageOrchestrator extends SequentialByPackageOrchestrator
